function [steps,step_sizes,dis] = kinesin_step_detection_v2(data,varargin)
%Version 2 using the Potts algorithm for step detection
%%% Takes position data from the tracking algorithm and
%%% finds the steps in the data. Localizes the step positions and generates
%%% a "denoised" path of the motor head.
%%% Data is taken from the guidata that is generated by the
%%% tracking_GUI in Fiona mode. Just data = guidata(figure) and then run 
%%% kinesin_step_detection(data).

%folder = 'C:\Data_Analysed\180921_11wt20EE_11\';
folder = 'C:\Data_Analysed\181228_wt_13\';
folder = 'C:\Data_Analysed\190812_sh_19\';

%setPLJavaPath
Potts_param_adj = 3*10^(-1); %higher for less sensitivity 3*10^-1  normally
run_num = 1;
run_range = 1; %size(data.values.runs{run_num},2);

if size(varargin,2) == 1
    do_plot = varargin{1};
elseif size(varargin,2) == 2
    do_plot = varargin{1};
    Potts_param_adj = varargin{2};
end

pixel_calibration = 93; %nm per pixel
if run_range(end) == 1
    run = data.values.runs{run_num}(6,:);
    frames = data.values.runs{run_num}(7,:);
else
    run = data.values.runs{run_num}(6,run_range);
    data.values.runs{run_num} = data.values.runs{run_num}(:,run_range);
    frames = 1:length(run);
end



%% old gliding-t-value-part
% for k = window_size+1:length(run)-window_size
%     xl = run(k-window_size:k-1);%     xl = run(k-window_size:k);
%     xr = run(k+1:k+window_size);%     xr = run(k:k+window_size);
%     t(k) = -(mean(xl) - mean(xr))/sqrt(1/window_size/2*(std(xl)^2+std(xr)^2)); % t(k-window_size)
% end
% % correct beginning
% t(1:window_size) = deal(t(window_size+1));
% 
% 
% %find peaks in t datatrac
% [~,peaklocs] = findpeaks(t,'MinPeakProminence',PeakProm);

%% Potts step detection
% try to find the Potts Parameter automatically from the noise in the data
Potts_param = std(run - smooth(run,20)')*Potts_param_adj;
%%% Override setting above
%Potts_para = Potts_param_adj;
%Potts_param = 0.05;
steps = minL2Potts(run, Potts_param);
steps = clean_up_steps(steps);

%% old step data generation
 %generate stepped datadata
%     if isempty(peaklocs) ~= 1 
%         for k = 1:length(peaklocs)
%             pos = peaklocs(k);
%             if k == 1 
%                 steps(1:pos) = mean(run(1:pos-1));
%             elseif k > 1
%                steps(peaklocs(k-1)+1:pos) = mean(run(peaklocs(k-1)+1:pos-1)); %steps(peaklocs(k-1):pos-1) = mean(run(peaklocs(k-1)+1:pos-1));
%             end
%         end
%         steps(pos:length(run)) = mean(run(pos:pos+window_size));
%     else
%        steps(1:length(run)-1) = mean(run(:));
%     end
    
%% data preperation
    step_sizes = diff(steps);
    step_sizes = step_sizes(step_sizes > 0);
  
    data.values.Pott_adj = Potts_param_adj;
    data.values.Pott = Potts_param;
    data.values.run_num = run_num;
    data.values.run_range = run_range;
    
    data.values.steps.detected = steps;
    data.values.steps.sizes = step_sizes;
    dis = sqrt(mean((data.values.steps.detected - run(1:length(data.values.steps.detected))).^2))*93;
    
    %% plotting
    if do_plot == 1
        handle_fig1 = figure('Units','normalized','Position',[0 0 1 0.8],'Color','w');
        handle_axes1 = subplot(2,2,[1,3]);
        uicontrol('Style','text','String',data.values.img_path,'Units','normalized','Position',[0 0.97 1 0.03],'FontSize',12,'Background','white')
        
        h_run_plot = plot(frames,run,'LineWidth',2);
        hold on
        step_plot_handle = plot(frames,steps,'.','Marker','none');
        run_point_plot = plot(frames,run,'.','Color',h_run_plot.Color,'Marker','none','MarkerSize',20);
        %plot(t/max(t(:))*max(run))
        hold off
        if exist('data','var')
            if isfield(data.values,'gauss_fit')
                handle_axes3 = subplot(2,2,2);
                handle_axes3.XLabel.String = 'r squared';
                title('r squared histogram')
                gauss_fit_data = cell2mat(data.values.gauss_fit); gauss_fit_data = gauss_fit_data(gauss_fit_data > 0.1);
                histogram(gauss_fit_data,'FaceColor','w')
                handle_axes3.XLim = [0.58 1];
                figure(handle_fig1)
                handle_axes2 = axes('Position',handle_axes1.Position,'Color','none');                
                plot(cell2mat(data.values.gauss_fit),'Color',[0.4 0.4 0.4],'Parent',handle_axes2)
                handle_axes2.Color = 'none';
                handle_axes2.YAxisLocation = 'right';
                handle_axes2.YLim = [0 1];
                handle_axes2.YLabel.String = 'r squared';
                uicontrol('Style','pushbutton','String','save','Callback',{@save_button,handle_fig1,data,folder},'Units','normalized','Position',[0 0.85 0.1 0.05]);
                uicontrol('Style','pushbutton','String','save for negative','Callback',{@save_button_negative,handle_fig1,data,folder},'Units','normalized','Position',[0 0.75 0.1 0.05]);
                uicontrol('Style','pushbutton','String','steps','Callback',{@step_button,step_plot_handle,data},'Units','normalized','Position',[0 0.65 0.1 0.05]);
                uicontrol('Style','pushbutton','String','positions','Callback',{@pos_button,run_point_plot,data},'Units','normalized','Position',[0 0.55 0.1 0.05]);
                handle_axes3.View = [90 -90];
            end
        end
        handle_axes1.YLabel.String = 'Distance in pixel';
        colors = get(groot,'DefaultAxesColorOrder');
        handle_axes1.YLabel.Color = colors(1,:);
        handle_axes1.XLabel.String = 'frame';
        handle_axes4 = subplot(2,2,4);
        scatter(1:length(step_sizes),step_sizes*pixel_calibration);
        hold on
        line([0 length(step_sizes)],[mean(step_sizes*pixel_calibration) mean(step_sizes*pixel_calibration)],'Color',colors(3,:));
        line([0 length(step_sizes)],[mean(step_sizes*pixel_calibration)-std(step_sizes*pixel_calibration) mean(step_sizes*pixel_calibration)-std(step_sizes*pixel_calibration)],'Color',colors(4,:),'LineStyle',':')
        line([0 length(step_sizes)],[mean(step_sizes*pixel_calibration)+std(step_sizes*pixel_calibration) mean(step_sizes*pixel_calibration)+std(step_sizes*pixel_calibration)],'Color',colors(4,:),'LineStyle',':')
        line([0 length(step_sizes)],[16 16],'Color',colors(5,:),'LineStyle','-.')
        line([0 length(step_sizes)],[8 8],'Color',colors(5,:),'LineStyle','-.')
        hold off
        handle_axes4.YLabel.String = 'step size [nm]';
        handle_axes4.YLim = [0 50];
        title(sprintf('mean step: %0.1f',mean(step_sizes*pixel_calibration)))
        axes(handle_axes1)
        handle_axes1.YLim = [min(run),max(run)*1.2];
        handle_axes1.YTick = min(run):0.2:max(run);
        handle_axes1.Color = 'none';
        handle_axes1.Box = 'off';
        handle_axes2.Box = 'off';
        set(zoom(handle_axes1),'ActionPostCallback',{@Zoom_Callback,handle_axes1,handle_axes2});
        set(pan(handle_fig1),'ActionPostCallback',{@Zoom_Callback,handle_axes1,handle_axes2});
    end

end


function save_button(source,~,handle_fig1,data,folder)
    delete(source)
    [path, name,ext] = fileparts(data.values.img_path);
    % prepare folder
    mkdir([folder datestr(datetime('now'),['yymmdd_' name])])
    % save figure
    savefig(handle_fig1,[folder datestr(datetime('now'),['yymmdd_' name]) '\figure.fig'],'compact')
    % save data
    data = data.values;
    data = rmfield(data,'stack');data = rmfield(data,'subpix_gauss');
    save([folder datestr(datetime('now'),['yymmdd_' name]) '\data.mat'],'data')
    % copy stack
    copyfile(data.img_path,[folder datestr(datetime('now'),['yymmdd_' name]) '\' name '.tif'])
end
 
function save_button_negative(source,~,handle_fig1,data,folder)
    delete(source)
    folder = [folder(1:end-1) '_neg\'];
    [path, name,ext] = fileparts(data.values.img_path);
    % prepare folder
    mkdir([folder datestr(datetime('now'),['yymmdd_' name])])
    % save figure
    savefig(handle_fig1,[folder datestr(datetime('now'),['yymmdd_' name]) '\figure.fig'])
    % save data
    data = data.values;
    data = rmfield(data,'stack');
    save([folder datestr(datetime('now'),['yymmdd_' name]) '\data.mat'],'data')
    % copy stack
    copyfile(data.img_path,[folder datestr(datetime('now'),['yymmdd_' name]) '\' name '.tif'])
end


function step_button(source,~,step_plot_handle,data)
    if strcmp(step_plot_handle.Marker,'none')
        step_plot_handle.Marker = '.';
    else
        step_plot_handle.Marker = 'none';
    end
end

function pos_button(source,~,run_point_plot,data)

    if strcmp(run_point_plot.Marker,'none')

        run_point_plot.Marker = '.';
    else
        run_point_plot.Marker = 'none';
    end
end

function Zoom_Callback(~,~,handle_axes1,handle_axes2)
drawnow
handle_axes2.XLim = handle_axes1.XLim;
end



function steps = clean_up_steps(steps)
% extract steps with only one or two positions
%compare value of step before and after and discard step, if step < 8nm
step_size_threshold = 8/94; %px


for i = 2:length(steps)-2  
   % detect peaks with width 2 or 1 up
   if steps(i) > steps(i-1) && steps(i+2) < steps(i)
       steps(i:i+1) = steps(i+2)*ones(2,1);
       %detect steps that are to small after peak detection
       if abs(steps(i) - steps(i-1)) < step_size_threshold
           steps(steps == steps(i)) = steps(i-1);
       end
   end
   
   %detect peaks down width 1 or 2
   if steps(i) < steps(i-1) && steps(i+2) > steps(i)
       steps(i:i+1) = steps(i-1)*ones(2,1);
   end
end
end